#!/usr/bin/perl

my $line;

#setup counters
my $data_sent=0;
my $data_recv=0;
my $data_size=0;
my $route_len=0;
my $route_opt=0;
my $rtr_sent=0;
my $rtr_fwd=0;
my $rtr_brdc=0;
my $rtr_size=0;
my %data_start;
my %data_end;
my %rtr_ttl;
my $arp_req=0;
my $arp_rep=0;
my $rtr_hello=0;

while (<STDIN>){
  $line = $_;
  # Count sent and received packages produced by CBR or TCP sources
  # and save their times (AGT - Agent Level Trace);
  if ($line =~ m/^s -t (\d+\.\d+).*?(-Nl AGT).*?-It (cbr|tcp).*?-Il (\d+).*?-Ii (\d+) /){
    $data_sent++;
    $data_start{$5} = $1;
    $data_size += $4;
    next;
  }
  if ($line =~ m/^r -t (\d+\.\d+).*?(-Nl AGT).*?-It (cbr|tcp).*?-Ii (\d+).*?-Pf (\d+).*?-Po (\d+)/){
    $data_recv++;
    $data_end{$4} = $1;
    #ccheck for optimal routes
    if ($6 < 16777215){ #skip if no optimal route available
      $route_len += $5;
      $route_opt += $6;
    }
    next;
  }
  # Count sent routing messages (RTR - Router Level Trace)
  if ($line =~ m/^s.*?(-Nl RTR).*?-Id (-?\d+)\..*?-It (DSDV|DSR|AODV|AODVUU|IMEP|NRLOLSR|message).*?-Il (\d+).*?-Iv (\d+)/){
    $rtr_brdc++ if($2 == -1); #destination broadcast?
    $rtr_sent++;
    $rtr_size+=$4;
    # what TTL?
    if($5 == 1){        # local only
      $rtr_ttl{1}++;
    }elsif($5 == 32){   # default
      $rtr_ttl{32}++;
    }else{              # other
      $rtr_ttl{x}++;
    }
		if($line =~ m/HELLO/){
      $rtr_hello++;
		}
    next;
  }
  # Count sent routing messages (RTR - Router Level Trace)
  if ($line =~ m/^f.*?(-Nl RTR).*?-Id (-?\d+)\..*?-It (DSDV|DSR|AODV|AODVUU|IMEP|NRLOLSR|message)/){
    $rtr_brdc++ if($2 == -1); #destination broadcast?
    $rtr_fwd++;
		if($line =~ m/HELLO/){
			#this shouldn't happen - HELLOs aren't forwarded but just to be sure
      $rtr_hello++;
		}
    next;
  }
  # Count Arp Request and Replies
  #if ($line =~ m/^s.*?(-Nl MAC).*?-Md ffffffff.*?-P arp -Po REQUEST/){
  if ($line =~ m/^s.*?(-Nl MAC).*?-P arp -Po REQUEST/){
    $arp_req++;
  }
  if ($line =~ m/^s.*?(-Nl MAC).*?-P arp -Po REPLY/){
    $arp_rep++;
  }
}

# calculate average delivery time
my $time=0;
my $avg_time=0;
foreach my $id (keys(%data_start)){
  $time = $data_end{$id} - $data_start{$id};
  if ($time > 0){
    $avg_time += $time;
  }
}
$avg_time = $avg_time/$data_recv;

# print data 
printdata('data_sent',$data_sent);
printdata('data_recv',$data_recv);
printdata('data_ratio',($data_recv/$data_sent)*100,'%.2f');
printdata('data_size',$data_size);
printdata('data_avgsz',$data_size/$data_sent,'%.2f');
printdata('route_len',$route_len);
printdata('route_opt',$route_opt);
printdata('route_ratio',($route_opt/$route_len)*100,'%.2f');
printdata('avg_time',$avg_time,'%.5f');
printdata('rtr_total',$rtr_sent+$rtr_fwd);
printdata('rtr_ratio',(($rtr_sent+$rtr_fwd)/$data_recv)*100,'%.2f');
printdata('rtr_sent',$rtr_sent);
printdata('rtr_fwd',$rtr_fwd);
printdata('rtr_brdc',$rtr_brdc);
printdata('rtr_ttl1',$rtr_ttl{1});
printdata('rtr_ttl32',$rtr_ttl{32});
printdata('rtr_ttlx',$rtr_ttl{x});
printdata('rtr_size',$rtr_size);
printdata('rtr_avgsz',$rtr_size/$rtr_sent,'%.2f');
printdata('rtr_szratio',($rtr_size/$data_size)*100,'%.2f');
printdata('arp_req',$arp_req);
printdata('arp_rep',$arp_rep);
printdata('rtr_hello',$rtr_hello);


sub printdata(){
  $name   = $_[0];
  $data   = $_[1];
  $format = $_[2];
  $format = '%d' if ($format eq '');
  printf("%12s:\t$format\n",$name,$data);
}
